package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"log"
	"net"
	"os"
	"strings"
	"sync"

	kyoketsu "git.aetherial.dev/aeth/kyoketsu/pkg"
)

func main() {

	local := flag.Bool("local", true, "set flag to false to run this in targeted remote mode")
	remoteAddrs := flag.String("ips", "", "comma seperated list of ip addresses to gather info about")
	iface := flag.String("iface", "eth0", "use this flag to specify the interface to autonomously use for scanning.")
	flag.Parse()

	if !*local {
		spAddr := strings.Split(*remoteAddrs, ",")
		addr, err := kyoketsu.GetAllRemoteAddresses(spAddr, 65535)
		if err != nil {
			log.Fatal(err)
		}
		b, err := json.Marshal(addr)
		if err != nil {
			log.Fatal(err)
		}
		os.WriteFile("test/slash16_ips.json", b, os.ModePerm)

		log.Printf("Starting scan on %v devices.\n", len(addr.Addr))
		var wg sync.WaitGroup
		for i := range addr.Addr {
			wg.Add(1)
			go func(idx int, wg *sync.WaitGroup) {

				out := kyoketsu.PortWalk(addr.Addr[idx].String(), kyoketsu.PORT_MAP)
				if len(out.ListeningPorts) > 0 {
					dns, _ := net.LookupAddr(out.IpAddress)
					out.Fqdn = strings.Join(dns, ", ")

					fmt.Printf("%+v\n", out)

				}

				wg.Done()
			}(i, &wg)
		}

		wg.Wait()

	} else {
		addr, err := kyoketsu.GetAllAddresses(*iface, 65535)
		if err != nil {
			log.Fatal(err)
		}
		log.Printf("Starting scan on %v devices.\n", len(addr.Addr))
		var wg sync.WaitGroup
		for i := range addr.Addr {
			wg.Add(1)
			go func(idx int, wg *sync.WaitGroup) {

				out := kyoketsu.PortWalk(addr.Addr[idx].String(), kyoketsu.PORT_MAP)
				if len(out.ListeningPorts) > 0 {
					fmt.Printf("%+v\n", out)

				}

				wg.Done()
			}(i, &wg)
		}

		wg.Wait()

	}
}
